unit hiAdjustControls;

interface

uses Windows,Kol,Share,Debug,Win;

const
  AlignName: array [0..5] of string = (
    'none','left','top','rigth','bottom','client' );
     
  PropNames: array[0..19] of string = (
    'left','top','width','height','align','color',
    'text','caption','strings','visible','enabled','checked','selected',
    'select','fontname','fontcolor','fontsize',
    'fontstyle','fontcharset','redraw' );

  ColorValues: array [0..46] of TColor = (
    clBlack,clMaroon,clGreen,clOlive,clNavy,clPurple,clTeal,clGray,
    clSilver,clRed,clLime,clYellow,clBlue,clFuchsia,clAqua,clWhite,
    clMoneyGreen,clSkyBlue,clCream,clMedGray,clActiveBorder,clActiveCaption,
    clAppWorkSpace,clBackground,clBtnFace,clBtnHighlight,clBtnShadow,    
    clBtnText,clCaptionText,clDefault,clGrayText,clHighlight,clHighlightText,
    clInactiveBorder,clInactiveCaption,clInactiveCaptionText,clInfoBk,
    clInfoText,clMenu,clMenuText,clNone,clScrollBar,cl3DDkShadow,
    cl3DLight,clWindow,clWindowFrame,clWindowText );

  ColorNames: array [0..46] of string = (
    'black','maroon','green','olive','navy','purple','teal','gray',
    'silver','red','lime','yellow','blue','fuchsia','aqua','white',
    'moneygreen','skyblue','cream','medgray','activeborder','activecaption',
    'appworkspace','background','btnface','btnhighlight','btnshadow',    
    'btntext','captiontext','default','graytext','highlight','highlighttext',
    'inactiveborder','inactivecaption','inactivecaptiontext','infobk',
    'infotext','menu','menutext','none','scrollbar','3ddkshadow',
    '3dlight','window','windowframe','windowtext' );

type
  THIAdjustControls = class(TDebug)
   private
    FListSet,
    FListIncludeName,
    FListExcludeName,
    FListIncludeProp,
    FListExcludeProp,
    FListProp: PStrList;
    fParameters,
    fIncludeNameList,
    fExcludeNameList,
    fIncludePropList,
    fExcludePropList: string;
    procedure SetParameters(const Value: string);
    procedure SetIncludeNameList(const Value: string);
    procedure SetExcludeNameList(const Value: string);
    procedure SetIncludePropList(const Value: string);
    procedure SetExcludePropList(const Value: string);
    function GetIntColor(const Value: string): integer;    
    function GetStrColor(const Value: integer): string;
    function isTrueName(Name: string): boolean;
    function isTrueProp(Prop: string): boolean;
   public
    _data_NameControl,
    _data_Parameters,
    _data_IncludeNameList,
    _data_ExcludeNameList,
    _data_IncludePropList,
    _data_ExcludePropList,
    _event_onSet,
    _event_onEnumNameControls,
    _event_onEndEnumNameControls,
    _event_onEnumParameters,
    _event_onEndEnumParameters: THI_Event;
    
    constructor Create;
    destructor Destroy; override;
    procedure _work_doSet(var _Data: TData; Index: word);
    procedure _work_doEnumNameControls(var _Data: TData; Index: word);
    procedure _work_doEnumParameters(var _Data: TData; Index: word);
    
    property _prop_Parameters: string read fParameters write SetParameters;
    property _prop_IncludeNameList: string read fIncludeNameList write SetIncludeNameList;
    property _prop_ExcludeNameList: string read fExcludeNameList write SetExcludeNameList;
    property _prop_IncludePropList: string read fIncludePropList write SetIncludePropList;
    property _prop_ExcludePropList: string read fExcludePropList write SetExcludePropList;
  end;
    
implementation

uses hiStr_Enum;

constructor THIAdjustControls.Create;
var
  i: integer;
begin
   inherited Create;
   FListSet := NewStrList;
   FListProp := NewStrList;
   FListIncludeName := NewStrList;
   FListExcludeName := NewStrList; 
   FListIncludeProp := NewStrList;
   FListExcludeProp := NewStrList;
   for i := 0 to High(PropNames) do 
     FListProp.Add(PropNames[i]);
end;

destructor THIAdjustControls.Destroy;
begin
   FListSet.Free;
   FListProp.Free;
   FListIncludeName.Free;
   FListExcludeName.Free;
   FListIncludeProp.Free;
   FListExcludeProp.Free;
   inherited;
end;

function THIAdjustControls.GetIntColor;
var
  i: integer;
begin
  for i := 0 to High(ColorNames) do
    if Value = ColorNames[i] then
    begin
      Result := ColorValues[i];
      exit;
    end;
  Result := str2int(Value);
end;

function THIAdjustControls.GetStrColor;
var
  i: integer;
begin
  for i := 0 to High(ColorValues) do
    if Color2RGB(Value) = Color2RGB(ColorValues[i]) then
    begin
      Result := ColorNames[i];
      exit;
    end;
  Result := int2str(Value);
end;

function THIAdjustControls.isTrueName(Name: string): boolean;
begin
  Result := ((FListIncludeName.IndexOf(Name) <> -1) or
            (FListIncludeName.IndexOf('all') <> -1)) and
            (FListExcludeName.IndexOf(Name) = -1);
end;

function THIAdjustControls.isTrueProp(Prop: string): boolean;
begin
  Result := ((FListIncludeProp.IndexOf(prop) <> -1) or
            (FListIncludeProp.IndexOf('all') <> -1)) and
            (FListExcludeProp.IndexOf(prop) = -1);
end;

//------------------------------- Set Lists ------------------------------------

procedure THIAdjustControls.SetParameters;
begin
  if fParameters = Value then exit; 
  fParameters := Value;
  FListSet.SetText(Value, false);
end;

procedure THIAdjustControls.SetIncludeNameList;
begin
  if fIncludeNameList = Value then exit;
  fIncludeNameList := Value;
  FListIncludeName.SetText(LowerCase(Value), false);
end;

procedure THIAdjustControls.SetExcludeNameList;
begin
  if fExcludeNameList = Value then exit;
  fExcludeNameList := Value;
  FListExcludeName.SetText(LowerCase(Value), false);
end;

procedure THIAdjustControls.SetIncludePropList;
begin
  if fIncludePropList = Value then exit;
  fIncludePropList := Value;
  FListIncludeProp.SetText(LowerCase(Value), false);
end;

procedure THIAdjustControls.SetExcludePropList;
begin
  if fExcludePropList = Value then exit;
  fExcludePropList := Value;
  FListExcludeProp.SetText(LowerCase(Value), false);
end;

//------------------------------------------------------------------------------

procedure THIAdjustControls._work_doSet;
var
  tmpControl: PControl;
  i, j, val: integer;
  _str, str, s_name, s_prop: string;

   procedure change(ind: integer);
   var
     k: integer;
   begin
     with tmpControl{$ifndef F_P}^{$endif} do
     case ind of
       0: Left := str2int(str);
       1: Top := str2int(str);
       2: Width := str2int(str);
       3: Height := str2int(str);
       4: begin
            if (LowerCase(str) = AlignName[0]) or (str ='0') then Align := caNone   else // none
            if (LowerCase(str) = AlignName[1]) or (str ='1') then Align := caLeft   else // left
            if (LowerCase(str) = AlignName[2]) or (str ='2') then Align := caTop    else // top         
            if (LowerCase(str) = AlignName[3]) or (str ='3') then Align := caRight  else // rigth 
            if (LowerCase(str) = AlignName[4]) or (str ='4') then Align := caBottom else // bottom 
            if (LowerCase(str) = AlignName[5]) or (str ='5') then Align := caClient;     // client 
            Invalidate;
          end;
       5: Color := GetIntColor(LowerCase(str)); 
   6,7,8: begin // text = caption = strings
            Replace(str, '|', #13#10);
            if isForm then
              SetWindowText(Handle, @str[1])
            else
              Text := str;
          end;
       9: Visible := boolean(ord(str2int(str)));
      10: Enabled := boolean(ord(str2int(str)));
   11,12: Checked := boolean(ord(str2int(str))); // checked = selected
      13: begin // select -  
            val := str2int(str);
            if (val < 0) or (val >= Count) then val := -1;
            CurIndex := val;
          end;
      14: Font.FontName := str;
      15: Font.Color := GetIntColor(LowerCase(str));
      16: Font.FontHeight := _hi_SizeFnt(str2int(str));
      17: begin
            Font.FontStyle := [];
            _str := LowerCase(str);
            for k := 1 to length(_str) do
            begin
              if _str[k] = 'b' then Font.FontStyle := Font.FontStyle + [fsBold] else
              if _str[k] = 'i' then Font.FontStyle := Font.FontStyle + [fsItalic] else
              if _str[k] = 'u' then Font.FontStyle := Font.FontStyle + [fsUnderline] else
              if _str[k] = 's' then Font.FontStyle := Font.FontStyle + [fsStrikeOut];
            end;
          end;
      18: Font.FontCharset := str2int(str);
      19: if str = 'true' then
            InvalidateRect(Handle, nil, true);
          else
            InvalidateRect(Handle, nil, false);          
     end;
   end;

begin
  SetParameters(ReadString(_Data, _data_Parameters, _prop_Parameters));
  SetIncludeNameList(ReadString(_Data, _data_IncludeNameList, _prop_IncludeNameList));
  SetExcludeNameList(ReadString(_Data, _data_ExcludeNameList, _prop_ExcludeNameList));
  SetIncludePropList(ReadString(_Data, _data_IncludePropList, _prop_IncludePropList));
  SetExcludePropList(ReadString(_Data, _data_ExcludePropList, _prop_ExcludePropList));

  for i := 0 to ListOfNameControls.Count - 1 do
  begin
    tmpControl := PControl(ListOfNameControls.Objects[i]);
    with tmpControl{$ifndef F_P}^{$endif}  do
    if isControl or isForm then
    begin
      for j := 0 to FListSet.Count - 1 do
      begin
        str := FListSet.Items[j];
        if str <> '' then
        begin
          s_name := LowerCase(fparse(str,'='));
          if ((s_name = LowerCase(ListOfNameControls.Items[i])) or (s_name = 'all')) and isTrueName(s_name) then
          begin
            s_prop := LowerCase(fparse(str,'='));
            if isTrueProp(s_prop) then change(FListProp.IndexOf(s_prop));
          end;
        end;
      end;
    end;
  end;
  _hi_OnEvent(_event_onSet);
end;

procedure THIAdjustControls._work_doEnumNameControls;
var
  i: integer;
begin
  SetIncludeNameList(ReadString(_Data, _data_IncludeNameList, _prop_IncludeNameList));
  SetExcludeNameList(ReadString(_Data, _data_ExcludeNameList, _prop_ExcludeNameList));

  for i := 0 to ListOfNameControls.Count - 1 do
    if isTrueName(ListOfNameControls.Items[i]) then
      _hi_onEvent(_event_onEnumNameControls, ListOfNameControls.Items[i]);
  _hi_onEvent(_event_onEndEnumNameControls);      
end;

procedure THIAdjustControls._work_doEnumParameters;
var
  r: real;
  sFontSize: integer;
  tmpControl: PControl;
  name, sFontStyle, sText: string;
  dtname, dtprop, dtval: TData;
  
  procedure outparameters(prop, value: string);
  begin
    if isTrueProp(prop) then
    begin   
      dtString(dtprop, prop);
      dtString(dtval, value);
      dtval.ldata := @dtprop; 
      _hi_onEvent_(_event_onEnumParameters, dtname);
    end;  
  end;

begin
  name := LowerCase(ReadString(_Data, _data_NameControl));
  if name = '' then exit;

  SetIncludeNameList(ReadString(_Data, _data_IncludeNameList, _prop_IncludeNameList));
  SetExcludeNameList(ReadString(_Data, _data_ExcludeNameList, _prop_ExcludeNameList));
  SetIncludePropList(ReadString(_Data, _data_IncludePropList, _prop_IncludePropList));
  SetExcludePropList(ReadString(_Data, _data_ExcludePropList, _prop_ExcludePropList));

  if not isTrueName(name) then exit;

  tmpControl := PControl(ListOfNameControls.Objects[ListOfNameControls.IndexOf(name)]);
  if not (tmpControl.isControl or tmpControl.isForm) then exit;

  dtString(dtname, name); 
  dtname.ldata := @dtval;
  with tmpControl{$ifndef F_P}^{$endif} do
  begin
    outparameters(PropNames[0], int2str(Left));              // left
    outparameters(PropNames[1], int2str(Top));               // top
    outparameters(PropNames[2], int2str(Width));             // width
    outparameters(PropNames[3], int2str(Height));            // height
    outparameters(PropNames[4], AlignName[ord(Align)]);      // align
    outparameters(PropNames[5], GetStrColor(Color));         // color
    sText := Text;
    Replace(sText, #13#10, '|');
    outparameters(PropNames[8], sText);                      // strings
    outparameters(PropNames[9], int2str(ord(Visible)));      // visible
    outparameters(PropNames[10], int2str(ord(Enabled)));     // enabled
    outparameters(PropNames[11], int2str(ord(Checked)));     // checked                    
    outparameters(PropNames[12], int2str(ord(Checked)));     // selected
    outparameters(PropNames[13], int2str(CurIndex));         // select    
    outparameters(PropNames[14], Font.FontName);             // fontname  
    outparameters(PropNames[15], GetStrColor(Font.Color));   // fontcolor
    r := ((Font.FontHeight * -72) - 36) / ScreenDPI;
    sFontSize := Trunc(r);
    if Frac(r) > 0 then Inc(sFontSize);
    outparameters(PropNames[16], int2str(sFontSize));        // fontsize
    sFontStyle := '';
    if fsBold in Font.FontStyle then  
      sFontStyle := sFontStyle + 'b';
    if fsItalic in Font.FontStyle then
      sFontStyle := sFontStyle + 'i';     
    if fsUnderline in Font.FontStyle then
      sFontStyle := sFontStyle + 'u';     
    if fsStrikeOut in Font.FontStyle then
      sFontStyle := sFontStyle + 's';     
    outparameters(PropNames[17], sFontStyle);                // fontstyle     
    outparameters(PropNames[18], int2str(Font.FontCharset)); // fontcharset
  end;
  _hi_onEvent(_event_onEndEnumParameters);   
end;

end.